package com.qxgmat.service;

import com.github.pagehelper.Page;
import com.nuliji.tools.AbstractService;
import com.nuliji.tools.Tools;
import com.nuliji.tools.exception.ParameterException;
import com.nuliji.tools.exception.SystemException;
import com.nuliji.tools.mybatis.Example;
import com.nuliji.tools.third.OauthData;
import com.qxgmat.data.constants.enums.status.DirectionStatus;
import com.qxgmat.data.dao.UserMapper;
import com.qxgmat.data.dao.entity.User;
import com.qxgmat.help.WechatHelp;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.Objects;

/**
 * Created by GaoJie on 2017/11/1.
 */
@Service
public class UsersService extends AbstractService {
    final public String PLATORM_WECHAT_NATIVE = "wechat_openid_wechat";
    final public String PLATORM_WECHAT_PC = "wechat_openid_pc";

    @Resource
    private UserMapper userMapper;

    @Resource
    private WechatHelp wechatHelp;

    public User Oauth(String code, String platform){
        OauthData data;
        switch(platform){
            case "wechat_pc":
                data = wechatHelp.oauthPc(code);
                break;
            case "wechat_native":
                data = wechatHelp.oauthNative(code);
                break;
            default:
                throw new ParameterException("第三方平台"+platform+"不支持");
        }
        // todo 登录已关联的用户账号
        User user = getByOpen(data.getOpenId(), platform);
        if (user == null){
            user = new User();
        }
        return user;
    }

    public User getByMobile(String mobile){
        User user = User.builder().mobile(mobile).build();
        return one(userMapper, user);
    }

    public User getByOpen(String openId, String platform){
        String platformField = "";
        switch(platform){
            case "wechat_pc":
                platformField = PLATORM_WECHAT_PC;
                break;
            case "wechat_native":
                platformField = PLATORM_WECHAT_NATIVE;
                break;
            default:
                throw new ParameterException("第三方平台"+platform+"不支持");
        }
        Example example = new Example(User.class);
        example.and(
                example.createCriteria()
                        .andEqualTo(platformField, openId)
        );
        return one(userMapper, example);
    }

    public boolean equalsPassword(User user, String password){
        return Objects.equals(user.getPassword(), Tools.stringMD5(password));
    }

    public boolean changePassword(User user, String password){
        User updateUser = User.builder().id(user.getId()).password(Tools.stringMD5(password)).build();
        int result = update(userMapper, updateUser);
        return result > 0;
    }

    public User getByInviteCode(String inviteCode){
        User user = new User();
        Example example = new Example(User.class);
        // 查找手机或邀请码
        example.and(
                example.createCriteria()
                        .orEqualTo("invite_code", inviteCode)
                        .orEqualTo("mobile", inviteCode)
        );
        return one(userMapper, example);
    }

    public boolean bindOpen(User user, User otherUser){
//        user.setTaobaoOpenid(otherUser.getTaobaoOpenid());
//        user.setWechatOpenid(otherUser.getWechatOpenid());
//        user.setRegisterType(otherUser.getRegisterType());
        int result = update(userMapper, user);
        return result > 0;
    }

    public User add(User user){
        if(user.getPassword() != null) user.setPassword(Tools.stringMD5(user.getPassword()));

        int result = insert(userMapper, user);
        user = one(userMapper, user.getId());
        if(user == null){
            throw new SystemException("用户添加失败");
        }
        return user;
    }

    public User edit(User user){
        User in = one(userMapper, user.getId());
        if(in == null){
            throw new ParameterException("用户不存在");
        }
        if(user.getPassword() != null) user.setPassword(Tools.stringMD5(user.getPassword()));
        int result = update(userMapper, user);
        return user;
    }

    public boolean delete(Number id){
        User in = one(userMapper, id);
        if(in == null){
            throw new ParameterException("用户不存在");
        }
        int result = delete(userMapper, id);
        return result > 0;
    }

    public User selectOne(Number id){
        User in = one(userMapper, id);

        if(in == null){
            throw new ParameterException("用户不存在");
        }
        return in;
    }

    public Page<User> select(int page, int pageSize){return select(userMapper, page, pageSize);
    }
    public Page<User> select(int page, int pageSize, String keyword, Boolean real, String order, DirectionStatus direction){
        Example example = new Example(User.class);
        if(!keyword.isEmpty())
            example.and(
                    example.createCriteria()
                            .orLike("id", "%"+keyword+"%")
                            .orLike("mobile", "%"+keyword+"%")
            );
        if (real != null)
            example.and(
                    example.createCriteria().andEqualTo("realStatus", real?1:0)
            );
        if(order.isEmpty()) order = "id";
        if (direction != null){
            switch(direction){
                case ASC:
                    example.orderBy(order).asc();
                    break;
                case DESC:
                default:
                    example.orderBy(order).desc();
            }
        } else {
            example.orderBy(order).desc();
        }
        return select(userMapper, example, page, pageSize);
    }


    public List<User> select(Collection ids){
        return select(userMapper, ids);
    }
}
